<!-- #include virtual="/Inc/NoteHeader.inc" --> 

<h1>Description.</h1>
Passing strings to and from COM is a very common practice but can be 
a little tricky if you don't allocate and deallocate correctly. All
strings are passed as <code>BSTR</code> and can be used with <code>CComBSTR</code>,
<code>_btr_t</code> and <code>CString</code> under most conditions. The following
should help answer some of the allocation issues associated
with using BSTRs.</p>

<h1>What is a BSTR</h1>

<p>Some might say that a <code>BSTR</code> is a pointer to an array of wchar_t , and in part
they would be correct. However it is more. It is a pointer to an array of
wchar_t that is managed by the OS so that it can be passed seamlessly across
process boundaries and exist longer then the process that created it. It
also contains the length of the string so it can contain embedded NULLs. For
these reasons it must be allocated using the <code>SysAllocString</code> functions and
freed with <code>SysFreeString</code>.</p>

<h1>The simple rules of [in] / [out]</h1>

<ul>
	<li>For [in] type parameters, client allocates and de-allocates the data.</li>
	<li>You may NOT modify BSTRs passed [in] to you.</li>
	<li>For [in,out] type parameters, client allocates, server re-allocates, and 
	    client does a final de-alocation.</li>
	<li>For [out] type parameters including [out,retval] parameters, the server 
	    allocates the data. <i>The client de-allocates output.</i></li>
</ul>

<h2>Passing BSTR into a function. [in]</h2>

<p>Client is responsible for allocating and deallocating the data.</p>
<pre>
    <font color="#006600">//In COM server implementation</font>
    STDMETHODIMP CWhereAmI<font color="#FF0000">:</font><font color="#FF0000">:</font>SetThing<font color="#FF0000">(</font> <font color="#006600">/*[in]*/</font>BSTR bstrThing <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        <font color="#006600">//I do not own it so I cannot modify it.</font>
        USES_CONVERSION<font color="#FF0000">;</font>
        CString sThing <font color="#FF0000">=</font> W2T<font color="#FF0000">(</font> bstrThing <font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">return</font> S_OK<font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
    ...
    ...
    ...
    <font color="#006600">//When I am using it</font>
    BSTR b1 <font color="#FF0000">=</font> SysAllocString<font color="#FF0000">(</font> L<font color="#FF00FF">"Hello"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    pSomeObj<font color="#FF0000">-</font><font color="#FF0000">></font>SetThing<font color="#FF0000">(</font> b1 <font color="#FF0000">)</font><font color="#FF0000">;</font>
    SysFreeString<font color="#FF0000">(</font> b1 <font color="#FF0000">)</font><font color="#FF0000">;</font>           
</pre>
 
<h2>Getting a BSTR from a function [out] or [out,retval].</h2>

<p>Client is responsible for providing and unallocated BSTR which
is populated by the function. The client must then deallocate the data.</p>
<pre>
    <font color="#006600">//In COM server implementation</font>
    STDMETHODIMP CWhereAmI<font color="#FF0000">:</font><font color="#FF0000">:</font>GetThing<font color="#FF0000">(</font> <font color="#006600">/*[out]*/</font> BSTR<font color="#FF0000">*</font> pbstrThing <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        <font color="#006600">//I create BSTR to return.</font>
        <font color="#FF0000">*</font>pbstrThing <font color="#FF0000">=</font> SysAllocString<font color="#FF0000">(</font> L<font color="#FF00FF">"Some returned stuff"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">return</font> S_OK<font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
    ...
    ...
    ...
    <font color="#006600">//When I am using it</font>
    BSTR b2 <font color="#FF0000">=</font> NULL<font color="#FF0000">;</font>
    pSomeObj<font color="#FF0000">-</font><font color="#FF0000">></font>GetThing<font color="#FF0000">(</font> <font color="#FF0000">&</font>b2 <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#006600">//Use the b2 now...</font>
    SysFreeString<font color="#FF0000">(</font> b2 <font color="#FF0000">)</font><font color="#FF0000">;</font>           
</pre>
 
<h2>Getting a BSTR from a function that is passed data in the same BSTR. [in,out]</h2>

<p>Client is responsible for allocating the input data and deallocating the output data.
However the server must free and allocate the data to modify. (Or use <code>SysReAllocString</code>)</p>
<pre>
    <font color="#006600">//In COM server implementation</font>
    STDMETHODIMP CWhereAmI<font color="#FF0000">:</font><font color="#FF0000">:</font>ReverseThing<font color="#FF0000">(</font> <font color="#006600">/*[in,out]*/</font> BSTR<font color="#FF0000">*</font> pbstrThing <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        USES_CONVERSION<font color="#FF0000">;</font>
        CString sThing <font color="#FF0000">=</font> W2T<font color="#FF0000">(</font> <font color="#FF0000">*</font>bstrThing <font color="#FF0000">)</font><font color="#FF0000">;</font>
        SysFreeString<font color="#FF0000">(</font> <font color="#FF0000">*</font>bstrThing <font color="#FF0000">)</font><font color="#FF0000">;</font>
        sThing.MakeReverse<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#006600">//I create BSTR to return.</font>
        <font color="#FF0000">*</font>pbstrThing <font color="#FF0000">=</font> sThing.AllocSysString<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">return</font> S_OK<font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
    ...
    ...
    ...
    <font color="#006600">//When I am using it</font>
    BSTR b3 <font color="#FF0000">=</font> SysAllocString<font color="#FF0000">(</font> L<font color="#FF00FF">"Foward Stuff"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    pSomeObj<font color="#FF0000">-</font><font color="#FF0000">></font>ReverseThing<font color="#FF0000">(</font> <font color="#FF0000">&</font>b3 <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#006600">//Use the b2 now...</font>
    <font color="#006600">//b3 should now contain "ffutS drawoF"</font>
    SysFreeString<font color="#FF0000">(</font> b3 <font color="#FF0000">)</font><font color="#FF0000">;</font>           
</pre>
 
<h1>What else can I use.</h1>

<p>Microsoft was kind enough to provide handy classes called <code>CComBSTR</code> and
<code>_btr_t</code> to make handling BSTRs a little simpler. Both classes wrap the BSTR
type to provide useful operators and methods. If you are doing more complex operations
these classes provide a richer set of features.</p>

<h1>BSTR in a little more depth (Words of wisdom)</h1>
 
<p>COM was designed to be a system for binary reuse.  If you've ever done any
programming working with standard DLL's, you would understand one of the
problems with them.  That is, if the memory manager for the dll, and the
memory manager for the main program are different (an example being debug
and release builds) and you allocate in one module and try to deallocate in
the other, your program would probably crash.  Using system functions for
doing the allocating and deallocating of memory guarantees that the memory
is managed the same for all BSTR's, no mater what type of memory manager you
program uses internally.  You never have any of the problems with allocating
in a release dll and deallocating in a debug program that you can have with
say straight C++ using DLL's.</p>

<p>Additionally, since BSTR's (and VARIANTs of course) are allocated in the
OS's memory space, not your process's memory space, they can be passed
between processes and even with a little background magic, can be passed
between computers (DCOM) without changing the programming.  The system
manages them and the address is valid for all processes in a given computer.
Also, unlike say new in C++, if your program terminates, the BSTR is still
valid even though all memory allocated by new would be automatically
returned to the system.  This is great if you are passing data between
processes, but can lead to memory leaks that can't be recovered short of
rebooting your system if you are a sloppy programmer and don't deallocate
the BSTR when you are done with it.</p>

<h1>Credits</h1>
<p>I would like to thank Vincent Minden for his help with this article.</p>

<h1>Further Reading</h1>

<p>A good MSDN article on this topic is 
<a href="http://msdn.microsoft.com/library/techart/msdn_article3.htm">Article 3. Strings the OLE Way</a></p>

<!-- #include virtual="/Inc/NoteFooter.inc" --> 




